package com.sample;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderErrors;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.event.process.DefaultProcessEventListener;
import org.drools.event.process.ProcessNodeLeftEvent;
import org.drools.event.process.ProcessNodeTriggeredEvent;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessInstance;
import org.drools.runtime.rule.Activation;
import org.drools.runtime.rule.AgendaFilter;

/**
 * This is a sample file to launch a process.
 */
public class RuleFlowTest {

  public static final void main(String[] args) {
    try {
      // load up the knowledge base
      KnowledgeBase kbase = readKnowledgeBase();
      final StatefulKnowledgeSession ksession = kbase
          .newStatefulKnowledgeSession();
      ksession.addEventListener(new DefaultProcessEventListener() {
        @Override
        public void afterNodeLeft(ProcessNodeLeftEvent event) {
          System.out.println("afterNodeLeft - "
              + event.getNodeInstance().getNodeName());
        }

        @Override
        public void afterNodeTriggered(ProcessNodeTriggeredEvent event) {
          System.out.println("afterNodeTriggered - "
              + event.getNodeInstance().getNodeName());
        }
      });
      // start a new process instance
      ProcessInstance process = ksession.startProcess("com.sample.ruleflow");
      ksession.fireAllRules(new AgendaFilter() {
        public boolean accept(Activation a) {
          if (a.getRule().getName().equals("ruleName")) {
            return false;
          }
          return true;
        }
      });
      // process should be completed
      System.out.println(process.getState());
      System.out.println("This should be true: " + (process.getState() == ProcessInstance.STATE_COMPLETED));
      System.out.println("This should be false: " + (process.getState() == ProcessInstance.STATE_ACTIVE));
    } catch (Throwable t) {
      t.printStackTrace();
    }
  }

  private static KnowledgeBase readKnowledgeBase() throws Exception {
    KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
    kbuilder.add(ResourceFactory.newClassPathResource("ruleflow.drl"),
        ResourceType.DRL);
    kbuilder.add(ResourceFactory.newClassPathResource("ruleflow.rf"),
        ResourceType.DRF);
    KnowledgeBuilderErrors errors = kbuilder.getErrors();
    if (errors.size() > 0) {
      for (KnowledgeBuilderError error : errors) {
        System.err.println(error);
      }
      throw new IllegalArgumentException("Could not parse knowledge.");
    }
    KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
    kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
    return kbase;
  }

}